from OpenGL.GL import *;
from OpenGL.GLU import *;
import sys;
import math;
import common;
import ac3d;
import os;
import texture;
import map;

Global_PlaneEquations=[[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]];

#concatinating two matrices
def matConcat( result, ma, mb ):
	mb00 = mb[0][0]; mb01 = mb[0][1];
	mb02 = mb[0][2]; mb03 = mb[0][3];
	mb10 = mb[1][0]; mb11 = mb[1][1];
	mb12 = mb[1][2];  mb13 = mb[1][3];
	mb20 = mb[2][0];  mb21 = mb[2][1];
	mb22 = mb[2][2];  mb23 = mb[2][3];
	mb30 = mb[3][0];  mb31 = mb[3][1];
	mb32 = mb[3][2];  mb33 = mb[3][3];

	for i in xrange(4):
		mai0 = ma[i][0];  mai1 = ma[i][1]; 
		mai2 = ma[i][2];  mai3 = ma[i][3];
		result[i*4+0] = mai0 * mb00 + mai1 * mb10 + mai2 * mb20 + mai3 * mb30;
		result[i*4+1] = mai0 * mb01 + mai1 * mb11 + mai2 * mb21 + mai3 * mb31;
		result[i*4+2] = mai0 * mb02 + mai1 * mb12 + mai2 * mb22 + mai3 * mb32;
		result[i*4+3] = mai0 * mb03 + mai1 * mb13 + mai2 * mb23 + mai3 * mb33;
	
#caliculate the distance of point p from plane.
def distanceFromPlane( peq, p ):
	return peq[0]*p[0] + peq[1]*p[1] + peq[2]*p[2] + peq[3];

def vecLen( plane ):
	sum = 0;
	for i in xrange(len(plane)):
		sum += plane[i]*plane[i];
	return math.sqrt(sum);
	
def Normalize( plane ):
	vlen = vecLen(plane);
	for i in xrange(len(plane)):
		plane[i] /= vlen;

#caliculating the plane equations of view volume in object coords
def calcViewVolumePlanes( ):
	mvm = glGetFloatv( GL_MODELVIEW_MATRIX );
	pm = glGetFloatv( GL_PROJECTION_MATRIX );

	ocCcMat = [1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1 ];
	matConcat( ocCcMat, mvm, pm ) 

	Global_PlaneEquations[0][0] = ocCcMat[3] - ocCcMat[0];
	Global_PlaneEquations[0][1] = ocCcMat[7] - ocCcMat[4];
	Global_PlaneEquations[0][2] = ocCcMat[11] - ocCcMat[8];
	Global_PlaneEquations[0][3] = ocCcMat[15] - ocCcMat[12];
	
	Global_PlaneEquations[1][0] = ocCcMat[3] + ocCcMat[0];
	Global_PlaneEquations[1][1] = ocCcMat[7] + ocCcMat[4];
	Global_PlaneEquations[1][2] = ocCcMat[11] + ocCcMat[8];
	Global_PlaneEquations[1][3] = ocCcMat[15] + ocCcMat[12];
	
	Global_PlaneEquations[2][0] = ocCcMat[3] + ocCcMat[1];
	Global_PlaneEquations[2][1] = ocCcMat[7] + ocCcMat[5];
	Global_PlaneEquations[2][2] = ocCcMat[11] + ocCcMat[9];
	Global_PlaneEquations[2][3] = ocCcMat[15] + ocCcMat[13];
	
	Global_PlaneEquations[3][0] = ocCcMat[3] - ocCcMat[1];
	Global_PlaneEquations[3][1] = ocCcMat[7] - ocCcMat[5];
	Global_PlaneEquations[3][2] = ocCcMat[11] - ocCcMat[9];
	Global_PlaneEquations[3][3] = ocCcMat[15] - ocCcMat[13];
	
	Global_PlaneEquations[4][0] = ocCcMat[3] + ocCcMat[2];
	Global_PlaneEquations[4][1] = ocCcMat[7] + ocCcMat[6];
	Global_PlaneEquations[4][2] = ocCcMat[11] + ocCcMat[10];
	Global_PlaneEquations[4][3] = ocCcMat[15] + ocCcMat[14];
	
	Global_PlaneEquations[5][0] = ocCcMat[3] - ocCcMat[2];
	Global_PlaneEquations[5][1] = ocCcMat[7] - ocCcMat[6];
	Global_PlaneEquations[5][2] = ocCcMat[11] - ocCcMat[10];
	Global_PlaneEquations[5][3] = ocCcMat[15] - ocCcMat[14];

	for plane in Global_PlaneEquations:
		Normalize(plane);

#test the bounding box of the object against the six planes
def culled( obj ):
	cntr = obj.center;
#	print obj.type+" ", cntr;
	for plane in Global_PlaneEquations:
	    #print plane[0]*cntr[0] + plane[1]*cntr[1] + plane[2]*cntr[2] + plane[3]; 
	    if plane[0]*(cntr[0]-obj.radius) + plane[1]*cntr[1] + plane[2]*cntr[2] + plane[3] < 0:
	#		return 1;
			if plane[0]*cntr[0] + plane[1]*(cntr[1]+obj.radius) + plane[2]*cntr[2] + plane[3] < 0:
				if plane[0]*cntr[0] + plane[1]*cntr[1] + plane[2]*(cntr[2]+obj.radius) + plane[3] < 0:
					if plane[0]*(cntr[0]+obj.radius) + plane[1]*cntr[1] + plane[2]*(cntr[2]+obj.radius) + plane[3] < 0:
						return 1;
	return 0;
	

#this function loads all the models in to the global
#array
def loadModels( ):
	print "loading models";
	tree = ac3d.ACObject();	
	tree.load(os.path.join("models","gametrunk.ac"));
	land = ac3d.ACObject();
	land.load(os.path.join("models","gamelandenemy.ac"));
	pplant = ac3d.ACObject( );
	pplant.load(os.path.join("models","powerplant.ac"));
	leaves = ac3d.ACObject( );
	leaves.load(os.path.join("models","gameleaves.ac"));
	air = ac3d.ACObject();
	air.load(os.path.join("models","gameairenemy.ac"));
	rocket = ac3d.ACObject();
	rocket.load(os.path.join("models","rocket.ac"));
	
	common.Global_ModelArray['tree'] = tree;
	common.Global_ModelArray['land'] = land;
	common.Global_ModelArray['pplant'] = pplant;
	common.Global_ModelArray['leaves'] = leaves;
	common.Global_ModelArray['air'] = air;
	common.Global_ModelArray['rocket'] = rocket;

	#loading texture for terrain( just a plane now )
	print "loading textures";
	common.Global_smokeTex = texture.loadTexture(os.path.join("textures","smoke.tga"));
	common.Global_grassTex = texture.loadTexture(os.path.join("textures","newgrass.tga"));
	common.Global_blueflareTex = texture.loadTexture(os.path.join("textures","blueflare.tga"));
	common.Global_flareAmmoTex = texture.loadTexture(os.path.join("textures","ammoflare.tga"));
	common.Global_rocketAmmoTex= texture.loadTexture(os.path.join("textures","rocketammo.tga"));

	for i in xrange(14):
		fname = os.path.join("textures","flame","f"+i.__str__()+".tga");
		id = texture.loadTexture(fname);
		common.Global_flameTex.append(id);
	
	#enabling stuff
	fogColor = [ 1, 1, 1 ];
	glEnable(GL_FOG);
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogfv( GL_FOG_COLOR, fogColor );
	glFogf(GL_FOG_START,  100);			
	glFogf(GL_FOG_END,    600);		
	glHint(GL_FOG_HINT, GL_NICEST);	

def drawHud( ):

    glColor3f( 1, 1, 1 );
#    cen = [ -30, -100];
#    glBindTexture( GL_TEXTURE_2D, common.Global_flareAmmoTex )
#    glBegin( GL_POLYGON );
#    glTexCoord2f(0,0);glVertex3f( -10+cen[0], 0+cen[1], 1 );
#    glTexCoord2f(1,0);glVertex3f( 10+cen[0], 0+cen[1], 1 );
#    glTexCoord2f(1,1);glVertex3f( 10+cen[0], 20+cen[1], 1 );
#    glTexCoord2f(0,1);glVertex3f( -10+cen[0], 20+cen[1], 1 );
#    glEnd( );

    cen = [ 0, -100];
    glBindTexture( GL_TEXTURE_2D, common.Global_flareAmmoTex )
    glBegin( GL_POLYGON );
    glTexCoord2f(0,0);glVertex3f( -10+cen[0], 0+cen[1], 1 );
    glTexCoord2f(1,0);glVertex3f( 10+cen[0], 0+cen[1], 1 );
    glTexCoord2f(1,1);glVertex3f( 10+cen[0], 20+cen[1], 1 );
    glTexCoord2f(0,1);glVertex3f( -10+cen[0], 20+cen[1], 1 );
    glEnd( );

    cen = [ 30, -100];
    glBindTexture( GL_TEXTURE_2D, common.Global_rocketAmmoTex )
    glBegin( GL_POLYGON );
    glTexCoord2f(0,0);glVertex3f( -10+cen[0], 0+cen[1], 1 );
    glTexCoord2f(1,0);glVertex3f( 10+cen[0], 0+cen[1], 1 );
    glTexCoord2f(1,1);glVertex3f( 10+cen[0], 20+cen[1], 1 );
    glTexCoord2f(0,1);glVertex3f( -10+cen[0], 20+cen[1], 1 );
    glEnd( );

    glEnable(GL_ALPHA_TEST);
    glDisable(GL_TEXTURE_2D);
    #box arround cur gun
#    if common.Global_curGun == "default":
#	cen = [ -30, -100 ];
    if common.Global_curGun == "rocketGun":
	cen = [ 30, -100 ];
    elif common.Global_curGun == "flareGun":
	cen = [ 0, -100 ];
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    glBegin( GL_POLYGON );
    glVertex3f( -10+cen[0], 0+cen[1], 2 );
    glVertex3f( 10+cen[0], 0+cen[1], 2 );
    glVertex3f( 10+cen[0], 20+cen[1], 2 );
    glVertex3f( -10+cen[0], 20+cen[1], 2 );
    glEnd( );
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );

    cen = [ -100, -60 ];
    #Mini Map
    glPushMatrix( );
    glTranslatef( cen[0], cen[1], 0 );
    scalefactor = .03;
    glColor4f( 1, 1, 1, 1 );
    myloc = common.Global_Camera.m_vPosition;
    mylook = common.Global_Camera.m_vView;
    rotAngle = map.rot( (mylook.x-myloc.x), mylook.z-myloc.z );
    glRotatef( -rotAngle, 0, 0, 1 );

    glColor4f( 0, 0, 0, 1 );
    glBegin(GL_LINES);
    glVertex3f( 0, 30, 1);
    glVertex3f( 0, -30, 1);
    glVertex3f( -30, 0, 1);
    glVertex3f( 30, 0, 1);
    glEnd( );

    
    glColor4f( 0, 1, 0, .4 );
    glBegin( GL_POLYGON );
    for i in xrange(16):
	glVertex3f( +30*math.sin(i*22.5*math.pi/180.0), +30*math.cos(i*22.5*math.pi/180.0), 1);
    glEnd( );
    
    glColor4f( 1, 0, 0, .4 );
    glBegin( GL_POLYGON );
    for i in xrange(16):
	glVertex3f( +6*math.sin(i*22.5*math.pi/180.0), +6*math.cos(i*22.5*math.pi/180.0), 1);
    glEnd( );

    glColor4f( 1, 1, 1, 1 );
    glBegin(GL_POLYGON);
    glVertex3f(-1+myloc.x*scalefactor,-1-myloc.z*scalefactor,1);
    glVertex3f(+1+myloc.x*scalefactor,-1-myloc.z*scalefactor,1);
    glVertex3f(+1+myloc.x*scalefactor,+1-myloc.z*scalefactor,1);
    glVertex3f(-1+myloc.x*scalefactor,+1-myloc.z*scalefactor,1);
    glEnd( );

    for obj in common.Global_objArray:
	if obj.type == "land" or obj.type == "air" or obj.type == "flareammo" or obj.type =="rocketammo":
	    if obj.type == "land":
		    glColor4f( 1, 0, 0, 1 )
	    elif obj.type == "air":
		    glColor4f( 1, 0, 1, 1 ) 
	    elif obj.type == "flareammo" or obj.type =="rocketammo":
		    glColor4f( .2, .2, 1, common.Global_t );
		    common.Global_t ^= 0;
	    glBegin(GL_POLYGON);
	    glVertex3f(-1+obj.center[0]*scalefactor,-1-obj.center[2]*scalefactor,1);
	    glVertex3f(+1+obj.center[0]*scalefactor,-1-obj.center[2]*scalefactor,1);
	    glVertex3f(+1+obj.center[0]*scalefactor,+1-obj.center[2]*scalefactor,1);
	    glVertex3f(-1+obj.center[0]*scalefactor,+1-obj.center[2]*scalefactor,1);
	    glEnd( );
    glPopMatrix( );		

    #cross
    glColor4f( 0.5, 0.28, 0.28, 1.0 );
    glBegin( GL_TRIANGLES )
    d=1
    glVertex3f(-2,2+d,1);
    glVertex3f(2,2+d,1);
    glVertex3f(0,0+d,1);
    glVertex3f(2+d,2,1);
    glVertex3f(2+d,-2,1);
    glVertex3f(0+d,0,1);
    glVertex3f(2,-2-d,1);
    glVertex3f(-2,-2-d,1);
    glVertex3f(0,0-d,1);
    glVertex3f(0-d,0,1);
    glVertex3f(-2-d,2,1);
    glVertex3f(-2-d,-2,1);
    glEnd();

    #drawing the helth
    cen = [ 130, -80 ];
    glColor4f( 1, 1, 1, 1 );
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
    glBegin( GL_POLYGON );
    glVertex3f( cen[0],   cen[1]-5,  1 );
    glVertex3f( cen[0],   cen[1]+5, 1 );
    glVertex3f( cen[0]-60,cen[1]+5,  1 );
    glVertex3f( cen[0]-60,cen[1]-5,  1 );
    glEnd( );
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
    
    life = common.Global_objArray[0].health;
    height = 60*life/500.0;
    glColor4f( 1-life/500.0, life/500.0, 0, 1 ); 
    glBegin( GL_POLYGON );
    glVertex3f( cen[0],   cen[1]-5,  1 );
    glVertex3f( cen[0],   cen[1]+5, 1 );
    glVertex3f( cen[0]-height,cen[1]+5,  1 );
    glVertex3f( cen[0]-height,cen[1]-5,  1 );
    glEnd( );
    glEnable(GL_TEXTURE_2D);
    glDisable(GL_ALPHA_TEST);

def render( ):
	#drawing the terrain
	color= [1,1,1,1];
	#glDisable(GL_FOG);
	glMaterialfv( GL_FRONT, GL_AMBIENT, color );
	glBindTexture( GL_TEXTURE_2D, common.Global_grassTex );
	width=300
	w=width*2
	for i in xrange(-2,3):
	    for j in xrange(-2,3):
		glBegin( GL_POLYGON );
		glNormal3fv( [ 0, 1, 0] );
		glTexCoord2f( 0, 0 );glVertex3f( i*w-width, 0, j*w-width );
		glNormal3fv( [ 0, 1, 0] );
		glTexCoord2f( 10, 0 );glVertex3f( i*w+width, 0, j*w-width );
		glNormal3fv( [ 0, 1, 0] );
		glTexCoord2f( 10, 10 );glVertex3f( i*w+width, 0, j*w+width );
		glNormal3fv( [ 0, 1, 0] );
		glTexCoord2f( 0, 10 );glVertex3f( i*w-width, 0,j*w+width );
		glEnd( );
	#glEnable(GL_FOG);
	#drawing the objects
	calcViewVolumePlanes( );
	for obj in common.Global_objArray:
		if culled(obj) and not(obj.type=="pplant"):
			continue;
		obj.render( );

	for obj in common.Global_objMovArray:
		if culled(obj):
		    continue;
		obj.render( );
