from common import *;

def loadTexture( imageName = "nehe_wall.bmp" ):
	"""Load an image file as a 2D texture using PIL

	This method combines all of the functionality required to
	load the image with PIL, convert it to a format compatible
	with PyOpenGL, generate the texture ID, and store the image
	data under that texture ID.

	Note: only the ID is returned, no reference to the image object
	or the string data is stored in user space, the data is only
	present within the OpenGL engine after this call exits.
	"""
	try:
		im = open(imageName)
	except:
		print "Could not open ", imageName;
	try:
		# get image meta-data (dimensions) and data
		ix, iy, image = im.size[0], im.size[1], im.tostring("raw", "RGBA", 0, -1)
	except SystemError:
		# has no alpha channel, synthesize one, see the
		# texture module for more realistic handling
		ix, iy, image = im.size[0], im.size[1], im.tostring("raw", "RGBX", 0, -1)

	# generate a texture ID
	ID= glGenTextures(1)
	# make it current
	glBindTexture(GL_TEXTURE_2D, ID)
	glPixelStorei(GL_UNPACK_ALIGNMENT,1)
	# copy the texture into the current texture ID

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
#	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL)

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, ix, iy, 0, GL_RGBA, GL_UNSIGNED_BYTE, image)
	
	# re-select our texture, could use other generated textures
	# if we had generated them earlier...
	#glBindTexture(GL_TEXTURE_2D, ID)

	# return the ID for use
	return ID

